@@ -396,11 +396,7 @@ void sh_init (void)
 
   /* The paths.
    */
-#if defined (SH_WITH_SERVER)
-  sl_strlcpy (sh.conf.path,  DEFAULT_YULE_CONFIGFILE,    SH_PATHBUF);
-#else
   sl_strlcpy (sh.conf.path,  DEFAULT_CONFIGFILE,    SH_PATHBUF);
-#endif
   sh.conf.hash[0] = '\0';
   sl_strlcpy (sh.data.path,  DEFAULT_DATA_FILE,     SH_PATHBUF);
   sh.data.hash[0] = '\0';
@@ -468,16 +464,6 @@ void sh_init (void)
     sl_strlcpy (sh.srvexport.name,  DEFAULT_LOGSERVER, SH_PATHBUF);
 
 
-#if defined (SH_WITH_SERVER)
-  if (0 == strcmp (DEFAULT_YULE_ERRLOCK, _("NULL")))
-    sh.srvlog.alt[0] = '\0';
-  else
-    sl_strlcpy (sh.srvlog.alt,  DEFAULT_YULE_ERRLOCK,       SH_PATHBUF);
-  if (0 == strcmp (DEFAULT_YULE_ERRFILE, _("NULL")))
-    sh.srvlog.name[0] = '\0';
-  else
-    sl_strlcpy (sh.srvlog.name,  DEFAULT_YULE_ERRFILE,      SH_PATHBUF);
-#else
   if (0 == strcmp (DEFAULT_ERRLOCK, _("NULL")))
     sh.srvlog.alt[0] = '\0';
   else
@@ -486,7 +472,6 @@ void sh_init (void)
     sh.srvlog.name[0] = '\0';
   else
     sl_strlcpy (sh.srvlog.name,  DEFAULT_ERRFILE,      SH_PATHBUF);
-#endif
 
   if (0 == strcmp (ALT_CONSOLE, _("NULL")))
     sh.srvcons.alt[0] = '\0';
@@ -690,13 +675,9 @@ static int * procdirSamhain ()
   for (i = 0; i < 65535; ++i) pidlist[i] = 0;
   i = 0;
 
-#ifdef SH_WITH_SERVER
-  if (0 != lstat(SH_INSTALL_YULE_PATH, &buf))
+  if (0 != stat(SH_INSTALL_PATH, &buf))
     return NULL;
-#else
-  if (0 != lstat(SH_INSTALL_PATH, &buf))
-    return NULL;
-#endif
+
   ino = (long) buf.st_ino;
     
   if (NULL == (dp = opendir("/proc")))
@@ -748,8 +729,16 @@ static int killprocSamhain (pid_t pid)
       kill (pid, SIGKILL);
       return (0);
     }
-  
-  return (1);
+  if (pid > 0)
+    {
+      if (errno == ESRCH)
+	return 7;
+      if (errno == EPERM)
+	return 4;
+      return 1;
+    }
+  else
+    return (7);
 }
 
 static int pidofSamhain (int flag)
@@ -761,11 +750,8 @@ static int pidofSamhain (int flag)
   pid_t       pid;
   struct stat buf;
  
-#ifdef SH_WITH_SERVER
-  fp = fopen (DEFAULT_YULE_ERRLOCK, "r");
-#else
   fp = fopen (DEFAULT_ERRLOCK, "r");
-#endif
+
   if (!fp)
     { if (errno != ENOENT) perror(_("fopen")); return 0; }
   if (NULL == fgets(line, 255, fp))
@@ -784,13 +770,10 @@ static int pidofSamhain (int flag)
    */
   if (flag == 1 && pid > 0 && 0 != kill(pid, 0) && errno == ESRCH)
     {
-#ifdef SH_WITH_SERVER
-      lstat (DEFAULT_YULE_ERRLOCK, &buf);
-      if (S_ISREG(buf.st_mode)) unlink(DEFAULT_YULE_ERRLOCK);
-#else
       lstat (DEFAULT_ERRLOCK, &buf);
       if (S_ISREG(buf.st_mode)) unlink(DEFAULT_ERRLOCK);
-#endif
+
+      pid = 0;
     }
   return pid;
 }
@@ -810,31 +793,18 @@ static int samhainctl(int ctl, int * argc, char * argv[])
   int         i;
 
 
-#ifdef SH_WITH_SERVER
-  fullpath = malloc(strlen(SH_INSTALL_YULE_PATH)+1);
-  if (fullpath == NULL)
-    { perror(_("malloc")); exit (4); }
-  else
-    strcpy(fullpath, SH_INSTALL_YULE_PATH);            /* known to fit  */
-
-  argp[0] = malloc(strlen(SH_INSTALL_YULE_PATH)+1);
-  if (argp[0] == NULL)
-    { perror(_("malloc")); exit (4); }
-  else
-    strcpy(argp[0], SH_INSTALL_YULE_PATH);             /* known to fit  */
-#else
   fullpath = malloc(strlen(SH_INSTALL_PATH)+1);
   if (fullpath == NULL)
-    { perror(_("malloc")); exit (4); }
+    { perror(_("malloc")); exit (1); }
   else
     strcpy(fullpath, SH_INSTALL_PATH);                 /* known to fit  */
 
   argp[0] = malloc(strlen(SH_INSTALL_PATH)+1);
   if (argp[0] == NULL)
-    { perror(_("malloc")); exit (4); }
+    { perror(_("malloc")); exit (1); }
   else
     strcpy(argp[0], SH_INSTALL_PATH);                  /* known to fit  */
-#endif
+
 
   for (times = 1; times < 32; ++times)  argp[times] = NULL;
 
@@ -844,7 +814,7 @@ static int samhainctl(int ctl, int * argc, char * argv[])
     {
       argp[times-1] = malloc(strlen(argv[times])+1);
       if (argp[times-1] == NULL)
-	{ perror(_("malloc")); exit (4); }
+	{ perror(_("malloc")); exit (1); }
       else
 	strcpy(argp[times-1], argv[times]);            /* known to fit  */
     }
@@ -854,7 +824,7 @@ static int samhainctl(int ctl, int * argc, char * argv[])
       envp[0] = 
 	malloc(strlen(getenv("TZ"))+1);
       if (envp[0] == NULL)
-	{ perror(_("malloc")); exit (4); }
+	{ perror(_("malloc")); exit (1); }
       else
 	strcpy(envp[0], getenv("TZ"));                   /* known to fit  */
     }
@@ -868,17 +838,22 @@ static int samhainctl(int ctl, int * argc, char * argv[])
     {
       pid = pidofSamhain(1);
 
+      if (pid != 0 && 0 == kill (pid, 0)) /* already started */
+	exit (0);
 
       pid = fork();
       switch (pid) {
       case -1:
 	perror(_("fork"));
-	exit (4);
+	exit (1);
       case  0:
 	close (0);
 	execve(fullpath, argp, envp);
-	perror(_("execve"));
-	_exit (4);
+	if (errno == EPERM)
+	  _exit(4);
+	else if (errno == ENOENT)
+	  _exit(5);
+	_exit (1);
       default:
 	times = 0;
 	while (times < 10) {
@@ -886,12 +861,15 @@ static int samhainctl(int ctl, int * argc, char * argv[])
 	  if (-1 == res)
 	    {
 	      perror(_("waitpid"));
-	      exit (4);
+	      exit (1);
 	    }
 	  else if (pid == res)
 	    {
 	      if (0 != WIFEXITED(status))
-		exit ( WEXITSTATUS(status) == 0 ? 0 : 1 );
+		{
+		  res = WEXITSTATUS(status);
+		  exit (res == 0 ? 0 : res );
+		}
 	      else
 		exit (1);
 	    }
@@ -907,8 +885,8 @@ static int samhainctl(int ctl, int * argc, char * argv[])
   if (ctl == 2)  /* stop */
     {
       pidlist = procdirSamhain ();
-      if (pid == 0 && NULL == pidlist)
-	return (2);
+      if (pid == 0 && NULL == pidlist) /* pid file not found */ 
+	return (0);
 	  
       status = 0;
       if (pid != 0)
@@ -923,32 +901,85 @@ static int samhainctl(int ctl, int * argc, char * argv[])
 	      ++i;
 	    }
 	}
-      return status;
+      if (status == 7)
+	return 0;
+      else
+	return status;
     }
 	
   if (ctl == 3)  /* reload */
     {
       if (pid == 0)
-	exit (2);
+        exit (7);
       if (0 == kill (pid, SIGHUP))
-	exit (0);
+        exit (0);
       else
-	exit (1);
+	{
+	  if (errno == EPERM)
+	    exit (4);
+	  if (errno == ESRCH)
+	    exit (7);
+	  exit (1);
+	}
     }
 
   if (ctl == 4)  /* status */
     {
       if (pid == 0)
-	exit (2);
+	exit (3);
       if (0 == kill (pid, 0))
 	exit (0);
       else
-	exit (1);
+	{
+	  if (errno == EPERM)
+	    exit (4);
+	  if (errno == ESRCH)
+	    exit (1);
+	}
     }
 
   exit (1); /* no exit handler installed yet */
 }
 
+#if defined (SH_WITH_CLIENT) || defined (SH_STANDALONE)
+#include "sh_schedule.h"
+static sh_schedule_t * FileSchedOne = NULL;
+static sh_schedule_t * FileSchedTwo = NULL;
+
+sh_schedule_t * sh_set_schedule_int (char * str, sh_schedule_t * FileSched, 
+				     int * status)
+{
+  SL_ENTER(_("sh_set_schedule_int"));
+  if (FileSched != NULL)
+    {
+      SH_FREE(FileSched);
+      FileSched = NULL;
+    }
+  FileSched = SH_ALLOC(sizeof(sh_schedule_t));
+  *status = create_sched(str, FileSched);
+  if (*status != 0)
+    {
+      SH_FREE(FileSched);
+      FileSched = NULL;
+    }
+  SL_RETURN(FileSched , _("sh_set_schedule_int"));
+}
+
+int sh_set_schedule_one (char * str)
+{
+  int status;
+  FileSchedOne = sh_set_schedule_int (str, FileSchedOne, &status);
+  return status;
+}
+int sh_set_schedule_two (char * str)
+{
+  int status;
+  FileSchedTwo = sh_set_schedule_int (str, FileSchedTwo, &status);
+  return status;
+}
+
+#endif
+
 /*******************************************************
  * 
  * Main program
@@ -970,6 +1001,9 @@ int main(int argc, char * argv[])
   float         st_1, st_2;
   int           status;
   long          cct = 0; /* main loop iterations */
+
+  int           flag_check_1 = 0;
+  int           flag_check_2 = 0;
 #endif
 
   unsigned long told;
@@ -1000,12 +1034,14 @@ int main(int argc, char * argv[])
         return (samhainctl (2, &argc, argv));
       else if (0 == strcmp(argv[1], _("reload")))
 	samhainctl (3, &argc, argv);
+      else if (0 == strcmp(argv[1], _("force-reload")))
+	samhainctl (3, &argc, argv);
       else if (0 == strcmp(argv[1], _("status")))
 	samhainctl (4, &argc, argv);
       else if (0 == strcmp(argv[1], _("restart")))
 	{
 	  res = samhainctl (2, &argc, argv);
-	  if (res == 0 || res == 2)
+	  if (res == 0 || res == 7)
 	    {
 	      samhainctl (1, &argc, argv);
 	    }
@@ -1079,9 +1115,11 @@ int main(int argc, char * argv[])
 
   /* --------  INIT  --------    
    */
+
+  /* drop privileges eventually
+   */
 #if defined(INET_SYSLOG) && defined(SH_WITH_SERVER)
   create_syslog_socket (S_TRUE);
-
   SL_REQUIRE(sl_policy_get_real(DEFAULT_IDENT) == SL_ENONE, 
 	     _("sl_policy_get_real(DEFAULT_IDENT) == SL_ENONE"));
 #else
@@ -1510,8 +1548,34 @@ int main(int argc, char * argv[])
       
       /* see whether its time to check files
        */
+      if      (sh.flag.checkSum == SH_CHECK_INIT)
+	{
+	  flag_check_1 = 1;
+	  if (FileSchedTwo != NULL) 
+	    flag_check_2 = 1;
+	}
+      else if (sh.flag.checkSum == SH_CHECK_CHECK)
+	{
+	  if (FileSchedOne == NULL)
+	    {
+	      /* use interval if we have no schedule
+	       */
+	      if (tcurrent - sh.fileCheck.alarm_last >= 
+		  sh.fileCheck.alarm_interval)
+		flag_check_1 = 1;
+	    }
+	  else
+	    {
+	      flag_check_1 = test_sched(FileSchedOne);
+	      if (FileSchedTwo != NULL) 
+		flag_check_2 = test_sched(FileSchedTwo);
+	      if (flag_check_2 == 1) 
+		flag_check_1 = 1;
+	    }
+	}
+
       if (sh.flag.checkSum != SH_CHECK_NONE &&
-	  tcurrent - sh.fileCheck.alarm_last >= sh.fileCheck.alarm_interval) 
+	  (flag_check_1 == 1 || flag_check_2 == 1))
 	{
 	  /* 
 	   * check directories and files
@@ -1522,10 +1586,14 @@ int main(int argc, char * argv[])
 
 	  TPT((0, FIL__, __LINE__, _("msg=<Check directories.>\n")))
 	  BREAKEXIT(sh_dirs_chk);
-	  sh.stat.dirs_checked  = sh_dirs_chk  (); 
+	  if (flag_check_1 == 1)
+	    sh.stat.dirs_checked  = sh_dirs_chk  (1); 
+	  if (flag_check_2 == 1)
+	    sh.stat.dirs_checked  = sh_dirs_chk  (2); 
 	  TPT((0, FIL__, __LINE__, _("msg=<Check files.>\n")))
 	  BREAKEXIT(sh_files_chk);
-	  sh.stat.files_checked = sh_files_chk ();
+	  if (flag_check_1 == 1)
+	    sh.stat.files_checked = sh_files_chk ();
 
 	  if (sig_terminate == 1)
 	    continue;
@@ -1533,12 +1601,20 @@ int main(int argc, char * argv[])
 	  /*
 	   * check for files not visited
 	   */
-	  TPT((0, FIL__, __LINE__, _("msg=<Check for missing files.>\n")))
-	  sh_hash_unvisited (ShDFLevel[SH_ERR_T_FILE]);
+	  if (flag_check_2 == 1 || FileSchedTwo == NULL)
+	    {
+	      TPT((0, FIL__, __LINE__, _("msg=<Check for missing files.>\n")))
+	      sh_hash_unvisited (ShDFLevel[SH_ERR_T_FILE]);
+	    }
 
 	  if (sig_terminate == 1)
 	    continue;
 
+	  /* 27.05.2002 avoid empty database
+	   */ 
+	  if      (sh.flag.checkSum == SH_CHECK_INIT)
+	    sh_hash_pushdata (NULL, NULL);
+
 	  /* reset
 	   */
 	  TPT((0, FIL__, __LINE__, _("msg=<Reset status.>\n")))
@@ -1547,6 +1623,9 @@ int main(int argc, char * argv[])
 	    continue;
 
 	  sh_files_reset ();
+	  flag_check_1 = 0;
+	  flag_check_2 = 0;
+
 	  if (sig_terminate == 1)
 	    continue;
 
@@ -1654,6 +1733,8 @@ int main(int argc, char * argv[])
   /*   ------  END  -----------
    */
 
+
+
   /*
    * cleanup
    */
