@@ -37,7 +37,7 @@ cat_entry msg_cat[] = {
 
   { MSG_MLOCK,       SH_ERR_WARN,    RUN,   N_("msg=\"Using insecure memory\"")},
   { MSG_W_SIG,       SH_ERR_WARN,    RUN,   N_("interface=\"sigaction\" msg=\"%s\" sig=\"%ld\"")},
-  { MSG_W_CHDIR,     SH_ERR_WARN,    RUN,   N_("interface=\"chdir\" msg=\"%s\" path=\"%s\"")},
+  { MSG_W_CHDIR,     SH_ERR_ERR,     RUN,   N_("interface=\"chdir\" msg=\"%s\" path=\"%s\"")},
 
   { MSG_MOD_FAIL,    SH_ERR_WARN,    RUN,   N_("msg=\"Module not initialized\" module=\"%s\" return_code=\"%ld\"")},
   { MSG_MOD_OK,      SH_ERR_INFO,    RUN,   N_("msg=\"Module initialized\" module=\"%s\"")},
@@ -115,6 +115,7 @@ cat_entry msg_cat[] = {
   { MSG_FI_LLNK,     SH_ERR_ALL,     FIL,   N_("msg=\"   >>>  %10s  %s\"")},
   { MSG_FI_MISS,     SH_ERR_ERR,     FIL,   N_("msg=\"POLICY MISSING\" path=\"%s\"")},
   { MSG_FI_ADD,      SH_ERR_ERR,     FIL,   N_("msg=\"POLICY ADDED\" path=\"%s\"")},
+  { MSG_FI_ADD2,     SH_ERR_ERR,     FIL,   N_("msg=\"POLICY ADDED\" path=\"%s\" %s")},
   { MSG_FI_CHAN,     SH_ERR_ERR,     FIL,   N_("msg=\"POLICY %s\" path=\"%s\" %s")},
   { MSG_FI_NODIR,    SH_ERR_ERR,     FIL,   N_("msg=\"POLICY NODIRECTORY\" path=\"%s\"")},
   { MSG_FI_DBEX,     SH_ERR_WARN,    FIL,   N_("msg=\"Signature database exists\" path=\"%s\"")},
@@ -156,7 +157,7 @@ cat_entry msg_cat[] = {
   { MSG_TCP_FINV ,   SH_ERR_SEVERE,  TCP,   N_("msg=\"Invalid request in pass %d\" host=\"%s\" request=\"%c%03o%c%03o%c%03o%c%03o\"")},
   { MSG_TCP_OKFILE,  SH_ERR_INFO,    TCP,   N_("msg=\"File transfer completed\" host=\"%s\"")},
   { MSG_TCP_OKMSG,   SH_ERR_ALL,     TCP,   N_("msg=\"Message transfer completed\" host=\"%s\"")},
-  { MSG_TCP_MSG,     SH_ERR_INET,    TCP,   N_("host=\"%s\" > %s </log>")},
+  { MSG_TCP_MSG,     SH_ERR_INET,    TCP,   N_("remote_host=\"%s\" > %s </log>")},
   { MSG_TCP_NEW,     SH_ERR_NOTICE,  TCP,   N_("msg=\"NEW CLIENT\" host=\"%s\"")},
   { MSG_TCP_ILL,     SH_ERR_SEVERE,  TCP,   N_("msg=\"Restart without prior exit\" host=\"%s\"")},
   { MSG_TCP_SYNC,    SH_ERR_SEVERE,  TCP,   N_("msg=\"Out of sync\" host=\"%s\"")},
@@ -178,7 +179,7 @@ cat_entry msg_cat[] = {
   { MSG_E_MMEM,      SH_ERR_ERR,     ERR,   N_("msg=\"Out of memory\"")},
 #else
   { MSG_MSTAMP,      SH_ERR_STAMP,   STAMP, N_("msg=\"Memory used:  max.=%ld, current=%ld\"")},
-  { MSG_MSTAMP2,     SH_ERR_STAMP,   STAMP, N_("msg=\"Allocated Blocks: %d, Free'd Blocks: %d\"")},
+  { MSG_MSTAMP2,     SH_ERR_STAMP,   STAMP, N_("msg=\"Allocated Blocks: %d, Freed Blocks: %d\"")},
   { MSG_E_MNULL,     SH_ERR_ERR,     ERR,   N_("msg=\"Dereferenced NULL pointer\" source_file=\"%s\" source_line=\"%d\"")},
   { MSG_E_MMEM,      SH_ERR_ERR,     ERR,   N_("msg=\"Out of memory\" source_file=\"%s\" source_line=\"%d\"")},
   { MSG_E_MREC,      SH_ERR_ERR,     ERR,   N_("msg=\"Free() on unrecorded block\" source_file=\"%s\" source_line=\"%d\"")},
@@ -293,7 +294,7 @@ cat_entry msg_cat[] = {
 
   { MSG_MLOCK,       SH_ERR_WARN,    RUN,   N_("msg=<Using insecure memory>")},
   { MSG_W_SIG,       SH_ERR_WARN,    RUN,   N_("msg=<%s>, interface=<sigaction>, signal=<%ld>")},
-  { MSG_W_CHDIR,     SH_ERR_WARN,    RUN,   N_("msg=<%s>, interface=<chdir>, path=<%s>")},
+  { MSG_W_CHDIR,     SH_ERR_ERR,     RUN,   N_("msg=<%s>, interface=<chdir>, path=<%s>")},
 
   { MSG_MOD_FAIL,    SH_ERR_WARN,    RUN,   N_("msg=<Module not initialized>, module=<%s>, return_code=<%ld>")},
   { MSG_MOD_OK,      SH_ERR_INFO,    RUN,   N_("msg=<Module initialized>, module=<%s>")},
@@ -436,7 +437,7 @@ cat_entry msg_cat[] = {
   { MSG_E_MMEM,      SH_ERR_ERR,     ERR,   N_("msg=<Out of memory>")},
 #else
   { MSG_MSTAMP,      SH_ERR_STAMP,   STAMP, N_("msg=<Memory used:  max.=%ld, current=%ld>")},
-  { MSG_MSTAMP2,     SH_ERR_STAMP,   STAMP, N_("msg=<Allocated Blocks: %d, Free'd Blocks: %d>")},
+  { MSG_MSTAMP2,     SH_ERR_STAMP,   STAMP, N_("msg=<Allocated Blocks: %d, Freed Blocks: %d>")},
   { MSG_E_MNULL,     SH_ERR_ERR,     ERR,   N_("msg=<Dereferenced NULL pointer>, source_file=<%s>, source_line=<%d>")},
   { MSG_E_MMEM,      SH_ERR_ERR,     ERR,   N_("msg=<Out of memory>, source_file=<%s>, source_line=<%d>")},
   { MSG_E_MREC,      SH_ERR_ERR,     ERR,   N_("msg=<Free() on unrecorded block>, source_file=<%s>, source_line=<%d>")},
