@@ -72,7 +72,8 @@ typedef struct dbins_ {
   char            syscall[16];
   char            ip[16];     
   char            tty[16];    
-  char            peer[64];   
+  char            peer[64];
+  char            fromhost[64];
   char            obj[1024];   
   char            interface[64];   
   char            ltime[64];   
@@ -85,6 +86,8 @@ typedef struct dbins_ {
 
   char            mode_old[16];
   char            mode_new[16];
+  char            attr_old[16];
+  char            attr_new[16];
   char            device_old[16];
   char            device_new[16];
   char            owner_old[9];
@@ -118,7 +121,7 @@ typedef struct dbins_ {
 static my_attr attr_tab[] = {
   { NULL, N_("sev"),         0,   1,    8, offsetof(struct dbins_, sev) },
   { NULL, N_("tstamp"),      0,   2,   16, offsetof(struct dbins_, time) },
-  { NULL, N_("host"),        0,   3,   64, offsetof(struct dbins_, host) },
+  { NULL, N_("remote_host"), 0,   3,   64, offsetof(struct dbins_, host) },
   { NULL, N_("msg"),         0,   4, 1024, offsetof(struct dbins_, msg) },
 
   { NULL, N_("path"),        0,   5, 1024, offsetof(struct dbins_, path)  },
@@ -188,10 +191,15 @@ static my_attr attr_tab[] = {
   { NULL, N_("port"),         0,  67,    0, 0  },
   { NULL, N_("return_code"),  0,  68,    0, 0 },
 
+  { NULL, N_("host"),         0,  69,   64, offsetof(struct dbins_, fromhost)  },
+  { NULL, N_("attr_old"),     0,  70,   16, offsetof(struct dbins_, attr_old)  },
+  { NULL, N_("attr_new"),     0,  71,   16, offsetof(struct dbins_, attr_new)  },
+
   { NULL, NULL,      0,  0, 0, 0 }
 };
 
 #define START_SEC_LONGS 51
+#define END_SEC_LONGS   68
 
 #if defined(HAVE_INT_32)
 typedef unsigned int uint32;
@@ -270,7 +278,7 @@ int sh_database_set_password (char * str)
  *  Postgresql stuff, completely untested, coded by the docs
  *
  ******************************************************************/
-#include "libpq-fe.h"
+#include <libpq-fe.h>
 int sh_database_query (char  * query, long * id)
 {
   char     * p;
@@ -348,7 +356,7 @@ int sh_database_query (char  * query, long * id)
 
 
 #ifdef WITH_MYSQL 
-#include "mysql.h"
+#include <mysql.h>
 int sh_database_query (char  * query, long * id)
 {
   char *     p;
@@ -474,14 +482,15 @@ long sh_database_entry (dbins * db_entry, long id)
 	sl_strlcpy (db_entry->host, sh.host.name, 64); 
     }
 
-  if (id > 0)
+  if (id >= 0)
     sprintf(num, "%ld", id);
 
+
   /*
   sl_snprintf (query, 1023,
 	       _("INSERT INTO %s VALUES (NULL,%s,\"%s\",\"%s\",\"%s\",\"%s\""),
 	       db_table,
-	       id > 0 ? num : _("NULL"),
+	       id >= 0 ? num : _("NULL"),
 	       db_entry->host,
 	       db_entry->time,
 	       db_entry->date,
@@ -489,7 +498,7 @@ long sh_database_entry (dbins * db_entry, long id)
   */
 
   sl_snprintf (values, 1023, _("(%s,%c%s%c,%c%s%c,%c%s%c,%c%s%c"),
-	       id > 0 ? num : _("NULL"),
+	       id >= 0 ? num : _("NULL"),
 	       '\'', db_entry->host,'\'', 
 	       '\'', db_entry->time,'\'', 
 	       '\'', db_entry->sev, '\'',
@@ -500,9 +509,12 @@ long sh_database_entry (dbins * db_entry, long id)
   sl_snprintf (columns, 1023, 
 	       _("(log_ref,log_host,log_time,log_sev,log_msg"));
 
-  if (attr_tab[0].inHash == 1) md5Update(&crc, db_entry->sev,  strlen(db_entry->sev));
-  if (attr_tab[1].inHash == 1) md5Update(&crc, db_entry->time, strlen(db_entry->time));
-  if (attr_tab[2].inHash == 1) md5Update(&crc, db_entry->host, strlen(db_entry->host));
+  if (attr_tab[0].inHash == 1) 
+    md5Update(&crc, db_entry->sev,  strlen(db_entry->sev));
+  if (attr_tab[1].inHash == 1) 
+    md5Update(&crc, db_entry->time, strlen(db_entry->time));
+  if (attr_tab[2].inHash == 1) 
+    md5Update(&crc, db_entry->host, strlen(db_entry->host));
   if (attr_tab[3].inHash == 1 && db_entry->msg[0] != '\0') 
     md5Update(&crc, db_entry->msg,  strlen(db_entry->sev));
 
@@ -529,13 +541,21 @@ long sh_database_entry (dbins * db_entry, long id)
 	  p = end;
 	  end = null_or_val(end,((char *)(db_entry)+attr_tab[i].off),&size,1);
 	  if (p != end)
-	    c_end = null_or_val (c_end, attr_tab[i].attr, &c_size,0);
+	    {
+	      /* 'host' is a reserved word in SQL
+	       */
+	      if (attr_tab[i].val == 69)
+		c_end = null_or_val (c_end, _("fromhost"), &c_size,0);
+	      else
+		c_end = null_or_val (c_end, attr_tab[i].attr, &c_size,0);
+	    }
 	  if (attr_tab[i].inHash == 1 && 
 	      ((char *)(db_entry)+attr_tab[i].off) != '\0')
 	    md5Update(&crc, ((char *)(db_entry)+attr_tab[i].off), 
 		    strlen((char *)(db_entry)+attr_tab[i].off));
 	}
-      else if (attr_tab[i].val >= START_SEC_LONGS)
+      else if (attr_tab[i].val >= START_SEC_LONGS &&
+	       attr_tab[i].val <= END_SEC_LONGS)
 	{
 	  sl_snprintf(end, (size-1), _(",\'%ld\'"), 
 		      db_entry->long_data[attr_tab[i].val-START_SEC_LONGS]);
@@ -761,12 +781,22 @@ char *  sh_database_parse (char * message, dbins * db_entry)
   goto parse;
 }
 
+static int enter_wrapper = 1;
+
+int set_enter_wrapper (char * str)
+{
+  return sh_util_flagval(str, &enter_wrapper);
+}
+
 int sh_database_insert (char * message)
 {
   dbins * db_entry;
   dbins * prev;
   dbins * curr;
   long    id = 0;
+#ifdef HOST_SWITCH
+  char  * temp[64];
+#endif
 
   SL_ENTER(_("sh_database_insert"));
 
@@ -777,17 +807,32 @@ int sh_database_insert (char * message)
    */
   sh_database_parse (message, db_entry);
 
-  /* enter the list into the database
+  /* Enter the list into the database. Actually, the list can only have
+   * two entries at most.
    */
   curr = db_entry;
   if (curr->next)
     {
       prev = curr->next;
-      id = sh_database_entry (prev, 0);
+#ifdef HOST_SWITCH
+      strncpy(temp, prev->host,       64);
+#endif
+      strncpy(prev->host, curr->host, 64);
+#ifdef HOST_SWITCH
+      strncpy(curr->host, temp,       64);
+#endif
+      id = sh_database_entry (prev, -1);
       SH_FREE(prev);
     }
 
-  sh_database_entry (curr, id);
+  if (id != 0)                       /* this is a server wrapper          */
+    {
+      if (enter_wrapper != 0)
+	sh_database_entry (curr, id);
+    }
+  else                                /* this is a generic server message */
+    sh_database_entry (curr, 0);
+    
   SH_FREE(curr);
 
   SL_RETURN(0, _("sh_database_insert"));
