@@ -459,6 +459,8 @@ int  sh_log_file (char *errmsg)
   char               * tmp = NULL;
   struct _sh_log_buf   log_msg;
   uid_t                uid;
+  static int           service_failure = 0;
+
 
   SL_ENTER(_("sh_log_file"));
 
@@ -469,7 +471,8 @@ int  sh_log_file (char *errmsg)
    */
   tmp  = sh_util_safe_name (sh.srvlog.name);
 
-  if (  0 !=  (status = tf_trust_check (sh.srvlog.name, SL_YESPRIV)))
+  if (  0 !=  (status = tf_trust_check (sh.srvlog.name, SL_YESPRIV))
+	&& service_failure == 0)
     sh_error_handle ((-1), FIL__, __LINE__, status, MSG_E_TRUST,
 		      (long) sh.effective.uid, tmp);
 
@@ -479,8 +482,9 @@ int  sh_log_file (char *errmsg)
       if (SL_ISERROR(fd))
 	{
 	  sl_get_euid(&uid);
-	  sh_error_handle ((-1), FIL__, __LINE__, status, MSG_E_ACCESS,
-			    (long) uid, tmp);
+	  if (service_failure == 0)
+	    sh_error_handle ((-1), FIL__, __LINE__, status, MSG_E_ACCESS,
+			     (long) uid, tmp);
 	  status = -1;
 	}
     }
@@ -492,8 +496,9 @@ int  sh_log_file (char *errmsg)
       if (status < 0)
 	{
 	  sl_get_euid(&uid);
-	  sh_error_handle ((-1), FIL__, __LINE__, status, MSG_LOCKED,
-			    (long) uid, tmp, sh.srvlog.alt);
+	  if (service_failure == 0)
+	    sh_error_handle ((-1), FIL__, __LINE__, status, MSG_LOCKED,
+			     (long) uid, tmp, sh.srvlog.alt);
 	  status = -1;
 	  sl_close(fd);
 	}
@@ -505,8 +510,9 @@ int  sh_log_file (char *errmsg)
       if (status < 0)
 	{
 	  sl_get_euid(&uid);
-	  sh_error_handle ((-1), FIL__, __LINE__, status, MSG_E_ACCESS,
-			    (long) uid, tmp);
+	  if (service_failure == 0)
+	    sh_error_handle ((-1), FIL__, __LINE__, status, MSG_E_ACCESS,
+			     (long) uid, tmp);
 	  status = -1;
 	  sl_close(fd);
 	}
@@ -514,21 +520,26 @@ int  sh_log_file (char *errmsg)
   
   if (status < 0)
     {
-      sh_error_handle ((-1), FIL__, __LINE__, status, MSG_SRV_FAIL,
-			_("logfile"), tmp);
+      if (service_failure == 0) {
+	sh_error_handle ((-1), FIL__, __LINE__, status, MSG_SRV_FAIL,
+			 _("logfile"), tmp);
+	service_failure = 1;
+      }
       SH_FREE(tmp);
       SL_RETURN ((-1), _("sh_log_file"));
     }
 
+  service_failure = 0;
+
 #ifdef SH_USE_XML
   if (NULL == errmsg)
     {
 #ifdef SH_STEALTH
-      sl_write_line (fd, N_("<trail>"), 7);
+      sl_write_line (fd, N_("</trail>"), 7);
       sl_write (fd, "\n", 1);
       sl_sync(fd);
 #else
-      sl_write_line (fd, _("<trail>\n"),  8);
+      sl_write_line (fd, _("</trail>\n"),  8);
       sl_sync(fd);
 #endif
       sl_close(fd);
@@ -598,8 +609,8 @@ int  sh_log_file (char *errmsg)
    */
   if (log_msg.msg[status-2] == '/')
     {
-      log_msg.msg[status-2] = '>';
-      log_msg.msg[status-1] = '<';
+      log_msg.msg[status-2] = '>'; /* ' ' FIX XML */
+      log_msg.msg[status-1] = '<'; /* '>' FIX XML */
       log_msg.msg[status]   = '\0';
     }
   else if (log_msg.msg[status-5] == '/')
@@ -678,11 +689,11 @@ int  sh_log_file (char *errmsg)
 #ifdef SH_USE_XML
   if (log_msg.timestamp[0] != '\0')
     sprintf(log_msg.sig,                            /* known to fit  */
-	    _("\nsig>%s%s</sig></log>\n"),
+	    _("\nsig>%s%s</sig></log>\n"),          /* <sig> FIX XML */
 	    log_msg.signature, log_msg.timestamp);
   else
     sprintf(log_msg.sig,                            /* known to fit  */
-	    _("\nsig>%s</sig></log>\n"), 
+	    _("\nsig>%s</sig></log>\n"),            /* <sig> FIX XML */
 	    log_msg.signature);
 #ifdef SH_STEALTH
   /* don't encode the line breaks (0 + last char)
