@@ -113,7 +113,12 @@ typedef struct dirstack_entry {
   struct dirstack_entry * next;
 } dirstack_t;
 
-static dirstack_t * dirList      = NULL;
+/* new 1.4.8
+ */
+static dirstack_t * dirListOne   = NULL;
+static dirstack_t * dirListTwo   = NULL;
+
+/* static dirstack_t * dirList      = NULL; */
 static dirstack_t * fileList     = NULL;
 
 static int        sh_files_fullpath  (char * testdir, char * d_name, 
@@ -187,8 +192,8 @@ unsigned long sh_files_chk ()
 				       (int *) &(ptr->reported));
 	  
 	  TPT(( 0, FIL__, __LINE__, 
-		_("msg=<filecheck complete: %s> status=<%d>\n"), 
-		tmp, status));
+		_("msg=<filecheck complete: %s> status=<%d> reported=<%d>\n"), 
+		tmp, status, ptr->reported));
 
 	  if (status == SH_FILE_UNKNOWN && ptr->reported == S_FALSE) 
 	    {
@@ -208,41 +213,61 @@ unsigned long sh_files_chk ()
 				   FIL__, __LINE__, 0,
 				   MSG_FI_FAIL,
 				   tmp);
-	      if (sh.flag.checkSum != SH_CHECK_INIT)
+#ifndef REPLACE_OLD
+	      /* this will tell that we have seen the file, and thus prevent
+	       * deletion from the database, resulting in an incomplete
+	       * message when the file reappears
+	       */
+	      if (sh.flag.checkSum != SH_CHECK_INIT) 
 		sh_hash_set_visited_true(ptr->name);
+#else
+	      if (sh.flag.checkSum != SH_CHECK_INIT) 
+		sh_hash_set_missing(ptr->name);
+#endif
 	      if (sh.flag.reportonce == S_TRUE)
 		ptr->reported = S_TRUE;
 	    }
-	  else
+	  else 
 	    {
-	      /* Was missing once, but is there again
+	      /* exists (status >= 0), but was missing (reported == TRUE)
 	       */
-#if 0
 	      if (status != SH_FILE_UNKNOWN && ptr->reported == S_TRUE)
 		{
 		  ptr->reported = S_FALSE;
-		  /* if (0 == sh_hash_have_it (ptr->name)) */
-		    sh_error_handle ((ptr->class == SH_LEVEL_ALLIGNORE) ? 
-				     ShDFLevel[ptr->class] : 
-				     ShDFLevel[SH_ERR_T_FILE],
-				     FIL__, __LINE__, 0, MSG_FI_ADD,
-				     tmp);
-		    /*
-		  else 
-		    sh_error_handle (SH_ERR_NOTICE, FIL__, __LINE__, 0,
-				     MSG_FI_ADD,
-				     tmp);
-		    */
 		}
-	      else
-#endif
-		if (status == SH_FILE_UNKNOWN)
+	      /* Catchall
+	       */
+	      else if (status == SH_FILE_UNKNOWN)
 		{
-		  sh_error_handle (SH_ERR_INFO, FIL__, __LINE__, 0,
-				   MSG_FI_FAIL,
-				   tmp);
-		  if (sh.flag.checkSum != SH_CHECK_INIT)
-		    sh_hash_set_visited_true(ptr->name);
+		  /* Thu Mar  7 15:09:40 CET 2002 Make sure missing file
+		   * is reported if ptr->reported == S_TRUE because the
+		   * file has been added.
+		   */
+		  if (sh_hash_have_it (ptr->name) >= 0)
+		    {
+		      sh_error_handle ((ptr->class == SH_LEVEL_ALLIGNORE) ? 
+				       ShDFLevel[ptr->class] : 
+				       ShDFLevel[SH_ERR_T_FILE],
+				       FIL__, __LINE__, 0, MSG_FI_MISS,
+				       tmp);
+#ifndef REPLACE_OLD
+		      if (sh.flag.checkSum != SH_CHECK_INIT) 
+			sh_hash_set_visited_true(ptr->name);
+#else
+		      /* delete from database
+		       */
+		      if (sh.flag.checkSum != SH_CHECK_INIT) 
+			sh_hash_set_missing(ptr->name);
+#endif
+		    }
+		  else
+		    {
+		      sh_error_handle (SH_ERR_INFO, FIL__, __LINE__, 0,
+				       MSG_FI_FAIL,
+				       tmp);
+		      if (sh.flag.checkSum != SH_CHECK_INIT)
+			sh_hash_set_visited_true(ptr->name);
+		    }
 		}
 	      ++fcount;
 	    }
@@ -275,39 +300,46 @@ int sh_files_delfilestack ()
   SL_RETURN(0, _("sh_files_delfilestack"));
 }
   
-int sh_files_setrec ()
+int sh_files_setrec_int (dirstack_t * ptr)
 {
-  dirstack_t * ptr = dirList;
-
   SL_ENTER(_("sh_files_setrec"));
-
   while (ptr != NULL) {
-
     if (ptr->rdepth < (-1) || ptr->rdepth > 99)
       {
 	ptr->rdepth = MaxRecursionLevel;
       }
     if (ptr->rdepth == (-1) && sh.flag.checkSum != SH_CHECK_INIT)
       hash_remove_tree (ptr->name);
-
     ptr = ptr->next;
   }
   SL_RETURN(0, _("sh_files_setrec"));
 }
 
-int sh_files_deldirstack ()
+int sh_files_setrec ()
 {
-  dirstack_t * old_ptr = dirList;
+  sh_files_setrec_int(dirListOne);
+  return sh_files_setrec_int(dirListTwo);
+}
+
+dirstack_t * sh_files_deldirstack_int (dirstack_t * ptr)
+{
+  dirstack_t * old_ptr = ptr;
 
   SL_ENTER(_("sh_files_deldirstack"));
-
   while (old_ptr != NULL) {
-    dirList = old_ptr->next;
+    ptr = old_ptr->next;
     SH_FREE (old_ptr->name);
     SH_FREE (old_ptr);
-    old_ptr = dirList;
+    old_ptr = ptr;
   }
-  SL_RETURN(0, _("sh_files_deldirstack"));
+  SL_RETURN(ptr, _("sh_files_deldirstack"));
+}
+
+int sh_files_deldirstack ()
+{
+  dirListOne = sh_files_deldirstack_int(dirListOne);
+  dirListTwo = sh_files_deldirstack_int(dirListTwo);
+  return 0;
 }
 
 void sh_files_reset()
@@ -324,13 +356,18 @@ void sh_files_reset()
 
 void sh_dirs_reset()
 {
-  dirstack_t * ptr = dirList;
+  dirstack_t * ptr = dirListOne;
 
   SL_ENTER(_("sh_dirs_reset"));
   while (ptr != NULL) {
     ptr->checked = 0;
     ptr = ptr->next;
   }
+  ptr = dirListTwo;
+  while (ptr != NULL) {
+    ptr->checked = 0;
+    ptr = ptr->next;
+  }
   SL_RET0(_("sh_dirs_reset"));
 }
 
@@ -731,14 +768,10 @@ int sh_files_pushfile (int class, char * str_s)
 
 /* ------ directories ----- */
 
-int sh_files_is_allignore (char * str)
+int sh_files_is_allignore_int (char * str, dirstack_t * ptr)
 {
-  dirstack_t * ptr;
-
   SL_ENTER(_("sh_files_is_allignore"));
 
-  ptr = dirList;
-  
   while (ptr)
     {
       if (0 == strcmp(str, ptr->name))
@@ -753,7 +786,16 @@ int sh_files_is_allignore (char * str)
   SL_RETURN( 0, _("sh_files_is_allignore"));
 }
 
-unsigned long sh_dirs_chk ()
+int sh_files_is_allignore (char * str)
+{
+  if (1 == sh_files_is_allignore_int(str, dirListOne))
+    return 1;
+  if (NULL == dirListTwo)
+    return 0;
+  return sh_files_is_allignore_int(str, dirListTwo);
+}
+
+unsigned long sh_dirs_chk (int which)
 {
   dirstack_t * ptr;
   dirstack_t * dst_ptr;
@@ -763,7 +805,10 @@ unsigned long sh_dirs_chk ()
   
   SL_ENTER(_("sh_dirs_chk"));
 
-  ptr = dirList;
+  if (which == 1)
+    ptr = dirListOne;
+  else
+    ptr = dirListTwo;
 
   while (ptr) 
     {
@@ -805,6 +850,8 @@ unsigned long sh_dirs_chk ()
 
 	  if (status < 0 && ptr->reported == S_FALSE) 
 	    {
+	      /* directory is missing
+	       */
 	      tmp = sh_util_safe_name (ptr->name);
 	      sh_error_handle ((ptr->class == SH_LEVEL_ALLIGNORE) ? 
 			       ShDFLevel[ptr->class] : 
@@ -816,11 +863,15 @@ unsigned long sh_dirs_chk ()
 	    } 
 	  else 
 	    {
-	      /* It is there again
+	      /* exists (status >= 0), but was missing (reported == TRUE)
 	       */
 	      if (status >= 0 && ptr->reported == S_TRUE)
 		{
 		  ptr->reported = S_FALSE;
+#if 0
+		  /* obsoleted (really?) by the mandatory sh_files_filecheck()
+		   * above, which will catch missing directories anyway
+		   */
 		  tmp = sh_util_safe_name (ptr->name);
 		  sh_error_handle ((ptr->class == SH_LEVEL_ALLIGNORE) ? 
 				   ShDFLevel[ptr->class] : 
@@ -828,9 +879,12 @@ unsigned long sh_dirs_chk ()
 				   FIL__, __LINE__, 0, MSG_FI_ADD,
 				   tmp);
 		  SH_FREE(tmp);
+#endif
 		}
 	      else if (status == SH_FILE_UNKNOWN)
 		{
+		  /* catchall
+		   */
 		  tmp = sh_util_safe_name (ptr->name);
 		  sh_error_handle (SH_ERR_INFO, FIL__, __LINE__, 0,
 				   MSG_FI_FAIL,
@@ -893,6 +947,16 @@ int sh_files_pushdir_allig (char * str_s)
   return (sh_files_pushdir (SH_LEVEL_ALLIGNORE, str_s));
 }
 
+static int which_dirList = 1;
+
+int set_dirList (int which)
+{
+  if (which == 2)
+    which_dirList = 2;
+  else
+    which_dirList = 1;
+  return 0;
+}
 
 int sh_files_push_dir_int (int class, char * tail, int len, int rdepth)
 {
@@ -913,10 +977,17 @@ int sh_files_push_dir_int (int class, char * tail, int len, int rdepth)
   new_item_ptr->checked        = S_FALSE;
   new_item_ptr->reported       = S_FALSE;
   new_item_ptr->childs_checked = S_FALSE;
-  new_item_ptr->next           = dirList;
-
-  dirList = new_item_ptr;
 
+  if (which_dirList == 1)
+    {
+      new_item_ptr->next           = dirListOne;
+      dirListOne                   = new_item_ptr;
+    }
+  else
+    {
+      new_item_ptr->next           = dirListTwo;
+      dirListTwo                   = new_item_ptr;
+    }
   SL_RETURN(0, _("sh_files_push_dir_int"));
 }
 
@@ -1083,7 +1154,6 @@ int sh_files_checkdir (int iclass, int idepth, char * iname)
   
   if (idepth < 0)
     {
-      /* fprintf(stderr, "FIXME: remove %s\n", iname); */
       /* hash_remove_tree (iname); */
       SL_RETURN((0), _("sh_files_checkdir"));
     }
@@ -1128,6 +1198,13 @@ int sh_files_checkdir (int iclass, int idepth, char * iname)
   status = sh_unix_getinfo (ShDFLevel[SH_ERR_T_DIR], 
 			    iname,
 			    &theFile, NULL);
+
+  if (status == -1)
+    {
+      SH_FREE(tmpname); 
+      SL_RETURN((-1), _("sh_files_checkdir"));
+    }
+
   if (theFile.c_mode[0] != 'd') 
     { 
       sh_error_handle (ShDFLevel[SH_ERR_T_DIR], FIL__, __LINE__, 0,
@@ -1234,12 +1311,18 @@ int sh_files_checkdir (int iclass, int idepth, char * iname)
 	{
 	  /* -- need to check the file itself --
 	   */
+	  if (dst_ptr && sh.flag.reportonce == S_TRUE)
+	    dummy = dst_ptr->reported;
 	  checkit = sh_files_filecheck (class_next, 
 					iname, 
 					thisEntry->d_name,
 					&dummy);
-	  if (checked_flag == S_FALSE)
+	  if (dst_ptr && checked_flag == S_FALSE)
 	    dst_ptr->checked = S_TRUE;
+	  /* Thu Mar  7 15:09:40 CET 2002 Propagate the 'reported' flag
+	   */
+	  if (dst_ptr && sh.flag.reportonce == S_TRUE)
+	    dst_ptr->reported = dummy;
 	}
 	  
       if      (checkit == SH_FILE_REGULAR)   
@@ -1258,7 +1341,7 @@ int sh_files_checkdir (int iclass, int idepth, char * iname)
 	      checked_flag  = -1;
 	      cchecked_flag = -1;
 
-	      dst_ptr     = dirList;
+	      dst_ptr     = dirListOne;
 	      while (dst_ptr) 
 		{
 		  if (0 == sl_strcmp(dst_ptr->name, tmpcat))
@@ -1277,6 +1360,28 @@ int sh_files_checkdir (int iclass, int idepth, char * iname)
 		  dst_ptr = dst_ptr->next;
 		}
 
+	      if (checked_flag == -1)
+		{
+		  dst_ptr     = dirListTwo;
+		  while (dst_ptr) 
+		    {
+		      if (0 == sl_strcmp(dst_ptr->name, tmpcat))
+			{
+			  TPT((0, FIL__, __LINE__, 
+			       _("msg=<%s -> recursion depth %d\n>"),
+			       dst_ptr->name, dst_ptr->rdepth));
+			  rdepth_next   = dst_ptr->rdepth;
+			  class_next    = dst_ptr->class;
+			  /* 28. Aug 2001 reversed
+			   */
+			  cchecked_flag = dst_ptr->childs_checked;
+			  checked_flag  = dst_ptr->checked;
+			  break;
+			}
+		      dst_ptr = dst_ptr->next;
+		    }
+		}
+
 	      if (cchecked_flag == S_FALSE)
 		{
 		  sh_files_checkdir (class_next, rdepth_next, tmpcat);
@@ -1341,6 +1446,8 @@ ShFileType sh_files_filecheck (int class, char * dirName, char * fileName,
   if (0 == (rand() % 2))
     (void) sh_derr();
 
+  /* fileName may be NULL if this is a directory
+   */
   if (dirName == NULL /* || fileName == NULL */)
     {
       sh_error_handle ((-1), FIL__, __LINE__, 0, MSG_FI_NULL);
@@ -1381,6 +1488,8 @@ ShFileType sh_files_filecheck (int class, char * dirName, char * fileName,
   theFile.check_mask = sh_files_maskof(class);
   theFile.reported   = (*reported);
 
+  TPT(( 0, FIL__, __LINE__, _("msg=<checking file: %s>\n"),  fullpath));
+
   status = sh_unix_getinfo ( (class == SH_LEVEL_ALLIGNORE) ? 
 			     ShDFLevel[class] : ShDFLevel[SH_ERR_T_FILE], 
 			     fileName,
@@ -1509,11 +1618,11 @@ static int check_file(char * name)
   SL_RETURN((-1), _("check_file"));
 }
   
-int sh_files_test_setup ()
+int sh_files_test_setup_int (dirstack_t * pdirL)
 {
   int dlen, flen;
 
-  dirstack_t * pdirL = dirList;  
+  /* dirstack_t * pdirL = dirList; */ 
   dirstack_t * pfilL = fileList;
 
   SL_ENTER(_("sh_files_test_setup"));
@@ -1560,6 +1669,11 @@ int sh_files_test_setup ()
   SL_RETURN((0), _("sh_files_test_setup"));
 }
       
-
+int sh_files_test_setup ()
+{
+  sh_files_test_setup_int (dirListOne);
+  sh_files_test_setup_int (dirListTwo);
+  return 0;
+}
 
 #endif
