@@ -25,7 +25,7 @@
 /* make sure state changes of a file are always reported, even
  *  with reportonlyonce=true
  */
-#define REPLACE_OLD
+/* #define REPLACE_OLD *//* moved to samhain.h */
 
 #include <stdlib.h>
 #include <string.h>
@@ -352,7 +352,7 @@ static void hash_unvisited (int j,
   if (p->next != NULL)
     hash_unvisited (j, p, p->next, level);
 
-  if (p->visited == S_FALSE && p->reported == S_FALSE 
+  if ((p->visited == S_FALSE || p->visited == 99) && p->reported == S_FALSE 
       && p->allignore == S_FALSE)
     {
       sl_set_suid(); 
@@ -368,9 +368,12 @@ static void hash_unvisited (int j,
 		level = ShDFLevel[SH_LEVEL_ALLIGNORE];
 	      SH_FREE(ptr);
 	    }
-	  tmp = sh_util_safe_name(p->fullpath);
-	  sh_error_handle (level, FIL__, __LINE__, 0, MSG_FI_MISS, tmp);
-	  SH_FREE(tmp);
+	  if (p->visited != 99)
+	    {
+	      tmp = sh_util_safe_name(p->fullpath);
+	      sh_error_handle (level, FIL__, __LINE__, 0, MSG_FI_MISS, tmp);
+	      SH_FREE(tmp);
+	    }
 	  if (sh.flag.reportonce == S_TRUE)
 	    {
 #ifdef REPLACE_OLD
@@ -634,6 +637,7 @@ sh_file_t *  sh_hash_getdataent (SL_TICKET fd, char * line)
 
   if (ft.mark != REC_MAGIC)
     {
+      /* fprintf(stderr, "FIXME ft.mark != REC_MAGIC\n"); */
       SH_FREE(p);
       SL_RETURN( NULL, _("sh_hash_getdataent"));
     }
@@ -770,6 +774,7 @@ void sh_hash_init ()
 		  sh_tiger_hash (file_path('C', 'R'), 
 				 TIGER_FD, 0),
 		  KEY_LEN+1);
+      sl_rewind (fd);
     }
   else 
 #endif
@@ -804,66 +809,72 @@ void sh_hash_init ()
 	  }
 	sl_rewind (fd);
 
+      } /* new 1.4.8 */
+
+
 #if defined(WITH_GPG) || defined(WITH_PGP)
-	/* extract the data and copy to temporary file
-	 */
-	fdTmp = open_tmp();
-	buf = (char *) SH_ALLOC(4096);
-	while ( (bufc = sh_unix_getline (fd, buf, 4095)) > 0) 
-	  {
-	    if (flag_pgp == S_FALSE &&
-		(0 == sl_strcmp(buf, _("-----BEGIN PGP SIGNED MESSAGE-----"))||
-		 0 == sl_strcmp(buf, _("-----BEGIN PGP MESSAGE-----")))
-		 )
-	      {
-		flag_pgp = S_TRUE;
-		sl_write_line(fdTmp, buf, bufc);
-		continue;
-	      }
+  /* new 1.4.8: also checked for server data */
 
-	    if (flag_pgp == S_TRUE && flag_nohead == S_FALSE)
-	      {
-		if (buf[0] == '\n')
-		  {
-		    flag_nohead = S_TRUE;
-		    sl_write(fdTmp, buf, 1);
-		    continue;
-		  }
-		else if (0 == sl_strncmp(buf, _("Hash:"), 5))
-		  {
-		    sl_write_line(fdTmp, buf, bufc);
-		    continue;
-		  }
-		else
-		  continue;
-	      }
+  /* extract the data and copy to temporary file
+   */
+  fdTmp = open_tmp();
 
-	    
-	    if (flag_pgp == S_TRUE && buf[0] == '\n')
-	      {
-		sl_write(fdTmp, buf, 1);
-	      }
-	    else if (flag_pgp == S_TRUE)
-	      {
-		sl_write_line(fdTmp, buf, bufc);
-	      }
+  buf = (char *) SH_ALLOC(4096);
+  while ( (bufc = sh_unix_getline (fd, buf, 4095)) > 0) 
+    {
+      if (flag_pgp == S_FALSE &&
+	  (0 == sl_strcmp(buf, _("-----BEGIN PGP SIGNED MESSAGE-----"))||
+	   0 == sl_strcmp(buf, _("-----BEGIN PGP MESSAGE-----")))
+	  )
+	{
+	  flag_pgp = S_TRUE;
+	  sl_write_line(fdTmp, buf, bufc);
+	  continue;
+	}
+      
+      if (flag_pgp == S_TRUE && flag_nohead == S_FALSE)
+	{
+	  if (buf[0] == '\n')
+	    {
+	      flag_nohead = S_TRUE;
+	      sl_write(fdTmp, buf, 1);
+	      continue;
+	    }
+	  else if (0 == sl_strncmp(buf, _("Hash:"), 5) ||
+		   0 == sl_strncmp(buf, _("NotDashEscaped:"), 15))
+	    {
+	      sl_write_line(fdTmp, buf, bufc);
+	      continue;
+	    }
+	  else
+	    continue;
+	}
+    
+      if (flag_pgp == S_TRUE && buf[0] == '\n')
+	{
+	  sl_write(fdTmp, buf, 1);
+	}
+      else if (flag_pgp == S_TRUE)
+	{
+	  sl_write_line(fdTmp, buf, bufc);
+	}
+      
+      if (flag_pgp == S_TRUE && 
+	  0 == sl_strcmp(buf, _("-----END PGP SIGNATURE-----")))
+	break;
+    }
+  SH_FREE(buf);
+  sl_close(fd);
+  fd = fdTmp;
+  sl_rewind (fd);
 
-	    if (flag_pgp == S_TRUE && 
-		0 == sl_strcmp(buf, _("-----END PGP SIGNATURE-----")))
-	      break;
-	  }
-	SH_FREE(buf);
-	sl_close(fd);
-	fd = fdTmp;
-	sl_rewind (fd);
-
-	/* Validate signature of open file.
-	 */
-	if (0 != sh_gpg_check_sign (0, fd, 2))
-	  aud_exit (FIL__, __LINE__, EXIT_FAILURE);
-	sl_rewind (fd);
+  /* Validate signature of open file.
+   */
+  if (0 != sh_gpg_check_sign (0, fd, 2))
+    aud_exit (FIL__, __LINE__, EXIT_FAILURE);
+  sl_rewind (fd);
 #endif
-      }
+  /* } new 1.4.8 check sig also for files downloaded from server */
 
   line = SH_ALLOC(SH_BUFSIZE+1);
 
@@ -927,18 +938,19 @@ void sh_hash_hashdelete ()
  ******************************************************************/ 
 void sh_hash_pushdata (file_type * buf, char * fileHash)
 {
-  sh_file_t * pp;
+  /* sh_file_t * pp; */
   static long p_count = 0;
-  long        pi;
+  /* long        pi; */
   char      * tmp;
 
-  unsigned short eo_m;
+  /* unsigned short eo_m; */
 
-  SL_TICKET fd;
+  static SL_TICKET fd = -1;
   sh_filestore_t p;
   static int isfirst = 1;
 
   struct stat sbuf;
+  int    status = 0;
 
   char fullpath[MAX_PATH_STORE+1];
   char linkpath[MAX_PATH_STORE+1];
@@ -950,7 +962,11 @@ void sh_hash_pushdata (file_type * buf, char * fileHash)
   linkpath[0] =  '-'; 
   linkpath[1] = '\0'; 
   fullpath[0] =  '-'; 
-  fullpath[1] = '\0'; 
+  fullpath[1] = '\0';
+
+  if (!buf) {
+    memset(&p, '\0', sizeof(sh_filestore_t));
+  }
 
   if (NULL == file_path('D', 'W') || 
       0 == sl_strcmp(file_path('D', 'W'), _("REQ_FROM_SERVER")))
@@ -973,54 +989,48 @@ void sh_hash_pushdata (file_type * buf, char * fileHash)
 
   if (sh.flag.update == S_FALSE)
     {
+      fd = -1;
       if ( SL_ISERROR(fd = sl_open_write(file_path('D', 'W'), SL_YESPRIV)))
 	SL_RET0(_("sh_hash_pushdata"));
       if ( SL_ISERROR(sl_forward(fd)))
 	SL_RET0(_("sh_hash_pushdata"));
     }
-  else
+  else /* update == TRUE */
     {
-      if ( SL_ISERROR(fd = sl_open_rdwr(file_path('D', 'W'), SL_YESPRIV)))
-	SL_RET0(_("sh_hash_pushdata"));
-      line = SH_ALLOC(SH_BUFSIZE+1);
-      if (SL_ISERROR(sh_hash_setdataent (fd, line)))
+      if (isfirst == 1)
 	{
-	  SH_FREE(line);
-	  SL_RET0(_("sh_hash_pushdata"));
-	}
-      for (pi = 0; pi < p_count; ++pi)
-	{
-	  pp = sh_hash_getdataent(fd, line);
-	  if (pp)
+	  if ( SL_ISERROR(fd = sl_open_rdwr(file_path('D', 'W'), SL_YESPRIV)))
+	    SL_RET0(_("sh_hash_pushdata"));
+	  line = SH_ALLOC(SH_BUFSIZE+1);
+	  if (SL_ISERROR(sh_hash_setdataent (fd, line)))
 	    {
-	      if (pp->fullpath)
-		SH_FREE(pp->fullpath);
-	      if (pp->linkpath)
-		SH_FREE(pp->linkpath);
-	      SH_FREE(pp);
+	      SH_FREE(line);
+	      SL_RET0(_("sh_hash_pushdata"));
 	    }
+	  SH_FREE(line);
 	}
-      SH_FREE(line);
     }
 	 
-  tmp = quote_string(buf->fullpath);
-  if (sl_strlen(tmp) <= MAX_PATH_STORE) 
-    {
-      sl_strlcpy(fullpath, buf->fullpath, MAX_PATH_STORE+1);
-      if (sl_strlen(buf->fullpath) < (MAX_PATH_STORE-3))
-	{
-	  fullpath[MAX_PATH_STORE-2] = '\0';
-	  fullpath[MAX_PATH_STORE-1] = '\n';
-	}
-    } 
-  else 
-    {
-      sl_strlcpy(fullpath, 
-		 sh_tiger_hash (buf->fullpath,
-				TIGER_DATA,sl_strlen(buf->fullpath)), 
-		 KEY_LEN+1);
-    }
-  SH_FREE(tmp);
+  if (buf != NULL) {
+    tmp = quote_string(buf->fullpath);
+    if (sl_strlen(tmp) <= MAX_PATH_STORE) 
+      {
+	sl_strlcpy(fullpath, buf->fullpath, MAX_PATH_STORE+1);
+	if (sl_strlen(buf->fullpath) < (MAX_PATH_STORE-3))
+	  {
+	    fullpath[MAX_PATH_STORE-2] = '\0';
+	    fullpath[MAX_PATH_STORE-1] = '\n';
+	  }
+      } 
+    else 
+      {
+	sl_strlcpy(fullpath, 
+		   sh_tiger_hash (buf->fullpath,
+				  TIGER_DATA,sl_strlen(buf->fullpath)), 
+		   KEY_LEN+1);
+      }
+    SH_FREE(tmp);
+  }
 
 #if defined(SH_STEALTH)
   sh_do_encode(fullpath, sl_strlen(fullpath));
@@ -1030,7 +1040,7 @@ void sh_hash_pushdata (file_type * buf, char * fileHash)
   sl_strlcpy(fullpath, tmp, sl_strlen(tmp)+1); 
   SH_FREE(tmp);
 
-  if (buf->c_mode[0] == 'l') 
+  if (buf != NULL && buf->c_mode[0] == 'l') 
     {  
       tmp = quote_string(buf->linkpath);
       if (sl_strlen(tmp) <= MAX_PATH_STORE) 
@@ -1054,63 +1064,66 @@ void sh_hash_pushdata (file_type * buf, char * fileHash)
       SH_FREE(tmp);
     }
 
-  p.mark = REC_MAGIC;
-  sl_strlcpy(p.c_mode,   buf->c_mode,   11);
-  sl_strlcpy(p.c_group,  buf->c_group,  GROUP_MAX+1);
-  sl_strlcpy(p.c_owner,  buf->c_owner,  USER_MAX+1);
-  sl_strlcpy(p.checksum, fileHash,      KEY_LEN+1);
+  if (buf != NULL) {
+    p.mark = REC_MAGIC;
+    sl_strlcpy(p.c_mode,   buf->c_mode,   11);
+    sl_strlcpy(p.c_group,  buf->c_group,  GROUP_MAX+1);
+    sl_strlcpy(p.c_owner,  buf->c_owner,  USER_MAX+1);
+    if (fileHash) {
+      sl_strlcpy(p.checksum, fileHash,      KEY_LEN+1);
+    }
 #if defined(__linux__)
-  sl_strlcpy(p.c_attributes, buf->c_attributes, 13);
+    sl_strlcpy(p.c_attributes, buf->c_attributes, 13);
 #endif
-
+    
 #if defined(SH_STEALTH)
-  sh_do_encode(p.c_mode,   sl_strlen(p.c_mode));
-  sh_do_encode(p.c_owner,  sl_strlen(p.c_owner));
-  sh_do_encode(p.c_group,  sl_strlen(p.c_group));
-  sh_do_encode(p.checksum, sl_strlen(p.checksum));
+    sh_do_encode(p.c_mode,   sl_strlen(p.c_mode));
+    sh_do_encode(p.c_owner,  sl_strlen(p.c_owner));
+    sh_do_encode(p.c_group,  sl_strlen(p.c_group));
+    sh_do_encode(p.checksum, sl_strlen(p.checksum));
 #if defined(__linux__)
-  sh_do_encode(p.c_attributes,   sl_strlen(p.c_attributes));
+    sh_do_encode(p.c_attributes,   sl_strlen(p.c_attributes));
 #endif
 #endif
-
+    
 #if defined(__linux__)
-  p.attributes  = (UINT32) buf->attributes;
+    p.attributes  = (UINT32) buf->attributes;
 #endif
-  p.linkmode    = (UINT32) buf->linkmode;
-  p.hardlinks   = (UINT32) buf->hardlinks;
-  p.dev   = (UINT64) buf->dev;
-  p.rdev  = (UINT64) buf->rdev;
-  p.mode  = (UINT32) buf->mode;
-  p.ino   = (UINT32) buf->ino;
-  p.size  = (UINT64) buf->size;
-  p.mtime = (UINT64) buf->mtime;
-  p.atime = (UINT64) buf->atime;
-  p.ctime = (UINT64) buf->ctime;
-  p.owner = (UINT32) buf->owner;
-  p.group = (UINT32) buf->group;
-
-
-  swap_32(&(p.mode));
-  swap_32(&(p.linkmode));
-  swap_64(&(p.dev));
-  swap_64(&(p.rdev));
-  swap_32(&(p.hardlinks));
-  swap_32(&(p.ino));
-  swap_64(&(p.size));
-  swap_64(&(p.atime));
-  swap_64(&(p.mtime));
-  swap_64(&(p.ctime));
-  swap_32(&(p.owner));
-  swap_32(&(p.group));
+    p.linkmode    = (UINT32) buf->linkmode;
+    p.hardlinks   = (UINT32) buf->hardlinks;
+    p.dev   = (UINT64) buf->dev;
+    p.rdev  = (UINT64) buf->rdev;
+    p.mode  = (UINT32) buf->mode;
+    p.ino   = (UINT32) buf->ino;
+    p.size  = (UINT64) buf->size;
+    p.mtime = (UINT64) buf->mtime;
+    p.atime = (UINT64) buf->atime;
+    p.ctime = (UINT64) buf->ctime;
+    p.owner = (UINT32) buf->owner;
+    p.group = (UINT32) buf->group;
+    
+    swap_32(&(p.mode));
+    swap_32(&(p.linkmode));
+    swap_64(&(p.dev));
+    swap_64(&(p.rdev));
+    swap_32(&(p.hardlinks));
+    swap_32(&(p.ino));
+    swap_64(&(p.size));
+    swap_64(&(p.atime));
+    swap_64(&(p.mtime));
+    swap_64(&(p.ctime));
+    swap_32(&(p.owner));
+    swap_32(&(p.group));
 #if defined(__linux__)
-  swap_32(&(p.attributes));
+    swap_32(&(p.attributes));
 #endif
 
 #ifdef OLD_BUG
-  swap_short(&(p.mark));
+    swap_short(&(p.mark));
 #else
-  p.mark = *(swap_short(&(p.mark)));
+    p.mark = *(swap_short(&(p.mark)));
 #endif
+  }
 
   /* write the start marker 
    */
@@ -1128,19 +1141,15 @@ void sh_hash_pushdata (file_type * buf, char * fileHash)
       isfirst = 0;
     }
       
-  sl_write      (fd,       &p, sizeof(sh_filestore_t));
-  sl_write_line (fd, fullpath,    sl_strlen(fullpath));
-  sl_write_line (fd, linkpath,    sl_strlen(linkpath));
-
-  if (sh.flag.update == S_TRUE)
-    {
-      eo_m = REC_MAGIC - 5;
-      sl_write      (fd,    &eo_m, sizeof(unsigned short));
-    }
+  status = sl_write      (fd,       &p, sizeof(sh_filestore_t));
+  status = sl_write_line (fd, fullpath,    sl_strlen(fullpath));
+  status = sl_write_line (fd, linkpath,    sl_strlen(linkpath));
 
   ++p_count;
 
-  sl_close (fd);
+  if (sh.flag.update != S_TRUE)
+    sl_close (fd);
+
   SL_RET0(_("sh_hash_pushdata"));
 }
 
@@ -1200,13 +1209,30 @@ static int sh_hash_set_visited_int (char * newname, int flag)
   
   if (p == NULL) 
     SL_RETURN((-1), _("sh_hash_set_visited_int"));
-  p->visited  = S_TRUE;
-  p->reported = flag;
-
+  if (flag == 99)
+    {
+      p->visited  = 99;
+      p->reported = S_FALSE;
+    }
+  else
+    {
+      p->visited  = S_TRUE;
+      p->reported = flag;
+    }
   SL_RETURN((0), _("sh_hash_set_visited_int"));
 }
 
 
+/* cause the record to be deleted without a 'missing' message
+ */
+int sh_hash_set_missing (char * newname)
+{
+  int i;
+  SL_ENTER(_("sh_hash_set_visited"));
+  i = sh_hash_set_visited_int(newname, 99);
+  SL_RETURN(i, _("sh_hash_set_visited"));
+}
+
 int sh_hash_set_visited (char * newname)
 {
   int i;
@@ -1310,6 +1336,10 @@ sh_file_t * sh_hash_push_int (file_type * buf, char * fileHash)
 
 #ifdef HAVE_INTTYPES_H
 #include <inttypes.h>
+#else
+#ifdef HAVE_STDINT_H
+#include <stdint.h>
+#endif
 #endif
 
 #ifndef PRIi64
@@ -1347,6 +1377,108 @@ char * my_size_format()
   SL_RETURN( form_rval, _("my_size_format"));
 }
 
+
+#ifdef SH_USE_XML
+char * all_items (file_type * theFile, char * fileHash)
+{
+  char timstr1c[32];
+  char timstr1a[32];
+  char timstr1m[32];
+
+  char * tmp_lnk;
+
+  char * tmp = SH_ALLOC(SH_BUFSIZE);
+  char * msg = SH_ALLOC(SH_BUFSIZE);
+
+  tmp[0] = '\0';
+  msg[0] = '\0';
+
+#if defined(__linux__)
+  sl_snprintf(tmp, SH_BUFSIZE, 
+	      _("mode_new=\"%s\" attr_new=\"%s\" imode_new=\"%ld\" iattr_new=\"%ld\" "),
+	      theFile->c_mode,
+	      theFile->c_attributes,
+	      (long) theFile->mode,
+	      (long) theFile->attributes
+	      );
+#else
+  sl_snprintf(tmp, SH_BUFSIZE, 
+	      _("mode_new=\"%s\" imode_new=\"%ld\" "),
+	      theFile->c_mode,
+	      (long) theFile->mode
+	      );
+#endif
+  sl_strlcat(msg, tmp, SH_BUFSIZE);
+
+
+  sl_snprintf(tmp, SH_BUFSIZE, 
+	      _("hardlinks_new=\"%ld\" "),
+	      (unsigned long) theFile->hardlinks);
+  sl_strlcat(msg, tmp, SH_BUFSIZE); 
+
+
+  sl_snprintf(tmp, SH_BUFSIZE,
+	      _("idevice_new=\"%ld\" "),
+	      (unsigned long) theFile->rdev
+	      );
+  sl_strlcat(msg, tmp, SH_BUFSIZE); 
+
+
+  sl_snprintf(tmp, SH_BUFSIZE,
+	      _("inode_new=\"%ld\" "),
+	      (unsigned long) theFile->ino);
+  sl_strlcat(msg, tmp, SH_BUFSIZE); 
+
+
+  sl_snprintf(tmp, SH_BUFSIZE, _("owner_new=\"%s\" iowner_new=\"%ld\" "),
+	      theFile->c_owner, (long) theFile->owner);
+  sl_strlcat(msg, tmp, SH_BUFSIZE); 
+
+
+  sl_snprintf(tmp, SH_BUFSIZE, _("group_new=\"%s\" igroup_new=\"%ld\" "),
+	      theFile->c_group, (long) theFile->group);
+  sl_strlcat(msg, tmp, SH_BUFSIZE); 
+
+
+  sl_snprintf(tmp, SH_BUFSIZE, my_size_format(),
+	      (UINT64) 0, (UINT64) theFile->size);
+  sl_strlcat(msg, tmp, SH_BUFSIZE); 
+
+
+  sl_strlcpy (timstr1c, sh_unix_gmttime (theFile->ctime),   32);
+  sl_snprintf(tmp, SH_BUFSIZE, _("ctime_new=\"%s\" "), timstr1c);
+  sl_strlcat(msg, tmp, SH_BUFSIZE); 
+
+  sl_strlcpy (timstr1a, sh_unix_gmttime (theFile->atime),   32);
+  sl_snprintf(tmp, SH_BUFSIZE, _("atime_new=\"%s\" "), timstr1a);
+  sl_strlcat(msg, tmp, SH_BUFSIZE); 
+
+  sl_strlcpy (timstr1m, sh_unix_gmttime (theFile->mtime),   32);
+  sl_snprintf(tmp, SH_BUFSIZE, _("mtime_new=\"%s\" "), timstr1m);
+  sl_strlcat(msg, tmp, SH_BUFSIZE); 
+
+  sl_snprintf(tmp, SH_BUFSIZE, 
+	      _("chksum_new=\"%s\" "),
+	      fileHash);
+  sl_strlcat(msg, tmp, SH_BUFSIZE); 
+
+  if (theFile->c_mode[0] == 'l')
+    {
+      tmp_lnk     = sh_util_safe_name(theFile->linkpath);
+      if (tmp_lnk)
+	{
+	  sl_snprintf(tmp, SH_BUFSIZE, _("link_new=\"%s\" "),
+		      tmp_lnk);
+	  SH_FREE(tmp_lnk);
+	  sl_strlcat(msg, tmp, SH_BUFSIZE);
+	} 
+    }
+  
+  SH_FREE(tmp);
+  return (msg);
+}
+#endif
+
 /*****************************************************************
  *
  * Compare a file with the database status.
@@ -1360,6 +1492,9 @@ void sh_hash_compdata (int class, file_type * theFile, char * fileHash)
   char * tmp_path;
   char * tmp_lnk;
   char * tmp_lnk_old;
+#ifdef SH_USE_XML
+  char * str;
+#endif
   char timstr1c[32];
   char timstr2c[32];
   char timstr1a[32];
@@ -1400,9 +1535,17 @@ void sh_hash_compdata (int class, file_type * theFile, char * fileHash)
 
 	}
 
+
       tmp = sh_util_safe_name(theFile->fullpath);
+#ifdef SH_USE_XML
+      str = all_items (theFile, fileHash);
+      sh_error_handle (ShDFLevel[class], FIL__, __LINE__, 0, MSG_FI_ADD2, 
+		       tmp, str);
+      SH_FREE(str);
+#else
       sh_error_handle (ShDFLevel[class], FIL__, __LINE__, 0, MSG_FI_ADD, 
 		       tmp);
+#endif
       SH_FREE(tmp);
 
       if (p)
@@ -1995,7 +2138,10 @@ int sh_hash_list_db (char * db_file)
       if (p != NULL)
 	sh_hash_list_db_entry (p); 
       else
-	break;
+	{
+	  /* fprintf(stderr, "FIXME: p == NULL\n"); */
+	  break;
+	}
     }
 
   if (line != NULL)
