@@ -1124,13 +1124,9 @@ static jmp_buf wait_timeout;
 
 static void sh_mail_alarmhandle (int mysignal)
 {
-  struct  sigaction  def_act;
-  def_act.sa_handler = SIG_IGN;
-
   if (mysignal == SIGALRM)   /* use 'signal' to avoid compiler warning */
     {  
       alarm(0);
-      sigaction (SIGALRM, &def_act, NULL);
       longjmp(wait_timeout, 1);
     }
 }
@@ -1150,8 +1146,16 @@ static int sh_mail_wait(int code, FILE * m_socket)
 
   struct  sigaction  def_act;
   struct  sigaction  new_act;
+  sigset_t           unblock;
+
+  sigemptyset(&unblock);
+  sigaddset  (&unblock, SIGALRM);
+
   def_act.sa_handler = SIG_IGN;
+
   new_act.sa_handler = sh_mail_alarmhandle;
+  sigemptyset( &new_act.sa_mask );       /* set an empty mask       */
+  new_act.sa_flags = 0;                  /* init sa_flags           */
 
   SL_ENTER(_("mail_wait"));
   
@@ -1159,6 +1163,9 @@ static int sh_mail_wait(int code, FILE * m_socket)
    */
   if (setjmp(wait_timeout) != 0)
     {
+      alarm(0);
+      sigaction (SIGALRM, &def_act, NULL);
+      sigprocmask(SIG_UNBLOCK, &unblock, NULL);
       TPT((0, FIL__, __LINE__, _("msg=<mail_wait: timeout>\n"))); 
       SL_RETURN( 0, _("mail_wait"));
     }
@@ -1168,7 +1175,7 @@ static int sh_mail_wait(int code, FILE * m_socket)
   /* timeout after 5 minutes
    */
   alarm(0);
-  sigaction (SIGALRM, &new_act, NULL);
+  sigaction (SIGALRM, &new_act, &def_act);
   alarm(time_wait);
 
   rcode = 0;
