@@ -255,7 +255,6 @@ int sh_readconf_read (void)
   sl_rewind (fd);
 #endif
 
-
   /* The system type, release, and machine.
    */
   sprintf(myident, _("%s:%s:%s"),                  /* known to fit  */
@@ -434,6 +433,20 @@ int sh_readconf_read (void)
 	  ++host_int;
       } 
 
+    /* ---  an %schedule directive ------------ */
+
+    else if (line[0] == '%' || (line[0] == '!' && line[1] == '%')) 
+      {
+#if defined (SH_WITH_CLIENT) || defined (SH_STANDALONE)
+	if (line[0] == '!' && 0 == sl_strcmp(&(line[2]), _("SCHEDULE_TWO")))
+	  set_dirList(1);
+	else if (0 == sl_strcmp(&(line[1]), _("SCHEDULE_TWO")))
+	  set_dirList(2);
+#else
+	;
+#endif
+      }
+	  
     /* ---  an $system directive -------------- */
 
     else if (line[0] == '$' || (line[0] == '!' && line[1] == '$')) 
@@ -655,6 +668,11 @@ typedef struct _cfg_options {
   int (*func)(char * opt);
 } cfg_options;
 
+#if defined (SH_WITH_CLIENT) || defined (SH_STANDALONE)
+extern int sh_set_schedule_one(char * str);
+extern int sh_set_schedule_two(char * str);
+#endif
+
 cfg_options ext_table[] = {
 #if defined(WITH_EXTERNAL)
   { N_("opencommand"),     SH_SECTION_EXTERNAL, SH_SECTION_NONE,  
@@ -702,6 +720,8 @@ cfg_options ext_table[] = {
     sh_error_set_database },
   { N_("databaseclass"),   SH_SECTION_LOG,      SH_SECTION_NONE,  
     sh_error_database_mask },
+  { N_("setdbservertstamp"), SH_SECTION_DATABASE,      SH_SECTION_NONE,  
+    set_enter_wrapper },
 #endif
 
 #if defined (SH_WITH_CLIENT) || defined (SH_STANDALONE)
@@ -739,6 +759,11 @@ cfg_options ext_table[] = {
   { N_("file"),           SH_SECTION_USER1,      SH_SECTION_NONE, 
     sh_files_pushfile_user1 },
 
+  { N_("filecheckscheduleone"), SH_SECTION_MISC,   SH_SECTION_NONE, 
+    sh_set_schedule_one },
+  { N_("filecheckscheduletwo"), SH_SECTION_MISC,   SH_SECTION_NONE, 
+    sh_set_schedule_two },
+
   { N_("setrecursionlevel"),  SH_SECTION_MISC,   SH_SECTION_NONE, 
     sh_files_setrecursion },
   { N_("checksumtest"),       SH_SECTION_MISC,   SH_SECTION_NONE, 
@@ -747,6 +772,8 @@ cfg_options ext_table[] = {
     sh_files_reportonce },
   { N_("reportfulldetail"),   SH_SECTION_MISC,   SH_SECTION_NONE, 
     sh_files_fulldetail },
+  { N_("uselocaltime"),       SH_SECTION_MISC,   SH_SECTION_NONE, 
+    sh_unix_uselocaltime },
 
   { N_("setnicelevel"),   SH_SECTION_MISC,   SH_SECTION_NONE, 
     sh_readconf_set_nice },
@@ -994,7 +1021,9 @@ static void sh_readconfig_line (char * line)
 			      _(modList[modnum].conf_table[modkey].the_opt),
 			      sl_strlen(modList[modnum].conf_table[modkey].the_opt)) == 0)
 		{
-		  modList[modnum].conf_table[modkey].func(c);
+		  if (0 != modList[modnum].conf_table[modkey].func(c))
+		    sh_error_handle ((-1), FIL__, __LINE__, 0, MSG_EINVALS,
+				     _(modList[modnum].conf_table[modkey].the_opt), c);
 		  if (!sl_is_suid())
 		    {
 		      TPT(( 0, FIL__, __LINE__, 
