@@ -74,6 +74,7 @@
 #include "sh_hash.h"
 #include "sh_unix.h"
 #include "sh_files.h"
+#include "sh_schedule.h"
 
 
 sh_rconf sh_suidchk_table[] = {
@@ -89,6 +90,10 @@ sh_rconf sh_suidchk_table[] = {
     N_("suidcheckinterval"),
     sh_suidchk_set_timer
   },
+  {
+    N_("suidcheckschedule"),
+    sh_suidchk_set_schedule
+  },
   {
     N_("suidcheckexclude"),
     sh_suidchk_set_exclude
@@ -117,6 +122,8 @@ static time_t  FileLimStart      = 0;
 static long    FileLimNum        = 0;
 static long    FileLimTotal      = 0;
 
+static sh_schedule_t * ShSuidchkSched = NULL;
+
 static char *
 filesystem_type (char * path, char * relpath, struct stat * statp);
 
@@ -241,7 +248,8 @@ int sh_suidchk_check_internal (char * iname)
 		  0 != strncmp (_("iso9660"), fs, 7) &&
 		  0 != strncmp (_("vfat"),    fs, 4) &&
 		  0 != strncmp (_("msdos"),   fs, 5) &&
-		  0 != strncmp (_("devfs"),   fs, 5)
+		  0 != strncmp (_("devfs"),   fs, 5) &&
+		  0 != strncmp (_("nosuid"),  fs, 6) 
 		  )
 		{
 		  /* fprintf(stderr, "%s: %s\n", fs, tmpcat); */
@@ -345,6 +353,10 @@ int sh_suidchk_end ()
  *************/
 int sh_suidchk_timer (unsigned long tcurrent)
 {
+  if (ShSuidchkSched != NULL)
+    {
+      return test_sched(ShSuidchkSched);
+    }
   if ((time_t) (tcurrent - lastcheck) >= ShSuidchkInterval)
     {
       lastcheck  = tcurrent;
@@ -430,6 +442,30 @@ int sh_suidchk_set_timer (char * c)
   SL_RETURN( 0, _("sh_suidchk_set_timer"));
 }
 
+int sh_suidchk_set_schedule (char * str)
+{
+  int status;
+
+  SL_ENTER(_("sh_suidchk_set_schedule"));
+
+  if (ShSuidchkSched != NULL)
+    {
+      SH_FREE(ShSuidchkSched);
+      ShSuidchkSched = NULL;
+    }
+  ShSuidchkSched = SH_ALLOC(sizeof(sh_schedule_t));
+  status = create_sched(str, ShSuidchkSched);
+  if (status != 0)
+    {
+      SH_FREE(ShSuidchkSched);
+      ShSuidchkSched = NULL;
+    }
+
+  SL_RETURN( status, _("sh_suidchk_set_schedule"));
+}
+
+
+
 int sh_suidchk_set_fps (char * c)
 {
   long val;
@@ -534,6 +570,7 @@ dirname (path)
    - replaced error()   by sh_error_handle()
    - replaced xstrdup() by sl_strdup()
    - replaced strstr()  by sl_strstr()
+   - some additions to recognize nosuid fs
 */
 
 /* modetype.h -- file type bits definitions for POSIX systems
@@ -874,7 +911,14 @@ filesystem_type_uncached (path, relpath, statp)
 	}
 
       if (dev == statp->st_dev)
-	type = mnt->mnt_type;
+	{
+	  /* check for the "nosuid" option
+	   */
+	  if (NULL == hasmntopt(mnt, "nosuid"))
+	    type = mnt->mnt_type;
+	  else
+	    type = "nosuid";
+	}
     }
 
   if (endmntent (mfp) == 0)
@@ -940,12 +984,22 @@ filesystem_type_uncached (path, relpath, statp)
 	}
     }
   else
-    type = fss.f_basetype;
+    {
+       type = fss.f_basetype;
+
+       /* patch by Konstantin Khrooschev <nathoo@co.ru> 
+	*/
+       if( fss.f_flag & ST_NOSUID )
+         type = "nosuid";
+    }
 #endif
 
 #ifdef FSTYPE_STATFS		/* 4.4BSD.  */
   struct statfs fss;
   char *p;
+#ifdef MNT_VISFLAGMASK
+  int flags;
+#endif
   /* extern char * dirname(const char *path); */
 
   if (path == NULL || relpath == NULL)
@@ -970,6 +1024,7 @@ filesystem_type_uncached (path, relpath, statp)
     }
   else
     {
+
 #ifdef MFSNAMELEN		/* NetBSD.  */
       /* MEMORY LEAK !!!
        *	 type = sh_util_strdup (fss.f_fstypename);
@@ -979,6 +1034,13 @@ filesystem_type_uncached (path, relpath, statp)
 #else
       type = fstype_to_string (fss.f_type);
 #endif
+#ifdef MNT_VISFLAGMASK
+      flags = fss.f_flags & MNT_VISFLAGMASK;
+      if (flags & MNT_NOSUID)
+#else 
+      if (fss.f_flags & MNT_NOSUID) 
+#endif
+         type = "nosuid";
     }
   if (p != relpath)
     SH_FREE (p);
