@@ -378,7 +378,7 @@ int connect_port (char * address, int port,
 	      if ((*errnum = sl_strcmp (host_name, host_entry->h_name)) != 0 || 
 		  sl_strlen(host_entry->h_name) == 0)
 		{ 
-		  if (errnum)
+		  if (*errnum)
 		    sl_strlcpy(ecall, _("strcmp"), SH_MINIBUF);
 		  else
 		    sl_strlcpy(ecall, _("strlen"), SH_MINIBUF);
@@ -965,10 +965,12 @@ char * get_client_data_file (char * peer, unsigned long * length)
 
 
   *length = 0;
+  SH_FREE(base);
   SL_RETURN(NULL, _("get_client_data_file"));
 
  lab1_end:
   *length = (unsigned long) buf.st_size;
+  SH_FREE(base);
   SL_RETURN(ret, _("get_client_data_file"));
   
 }
