@@ -1081,6 +1081,7 @@ int sh_unix_init(int goDaemon)
 {
   int    status;
   uid_t  uid;
+  pid_t  oldpid = getpid();
 
   SL_ENTER(_("sh_unix_init"));
 
@@ -1148,6 +1149,7 @@ int sh_unix_init(int goDaemon)
    */
   sh_unix_zeroenv();
 
+
   /* close all file descriptors, and 
    * open first three streams to /dev/null 
    */
@@ -1158,6 +1160,22 @@ int sh_unix_init(int goDaemon)
       retry_aud_dup(FIL__, __LINE__, 0); 
       retry_aud_dup(FIL__, __LINE__, 0);
 
+      /* --- wait until parent has exited ---
+       */
+      while (1 == 1)
+	{
+	  errno = 0;
+	  if (0 > aud_kill (FIL__, __LINE__, oldpid, 0) && errno == ESRCH)
+	    {
+	      break;
+	    }
+#ifdef HAVE_USLEEP
+	  usleep(100);
+#else
+	  sleep(1);
+#endif
+	}
+
       /* write PID file
        */
       status = sh_unix_testlock(NULL);
@@ -1423,6 +1441,19 @@ char * sh_unix_time (time_t thetime)
   SL_RETURN( _(deftime), _("sh_unix_time"));
 }
 
+static int sh_unix_use_localtime = S_FALSE;
+
+/* whether to use localtime for file timesatams in logs
+ */
+int sh_unix_uselocaltime (char * c)
+{
+  int i;
+  SL_ENTER(_("sh_unix_uselocaltime"));
+  i = sh_util_flagval(c, &(sh_unix_use_localtime));
+
+  SL_RETURN(i, _("sh_unix_uselocaltime"));
+}
+    
 char * sh_unix_gmttime (time_t thetime)
 {
 
@@ -1438,7 +1469,10 @@ char * sh_unix_gmttime (time_t thetime)
 
   SL_ENTER(_("sh_unix_gmttime"));
 
-  time_ptr   = gmtime (&thetime);
+  if (sh_unix_use_localtime == S_FALSE)
+    time_ptr   = gmtime (&thetime);
+  else
+    time_ptr   = localtime (&thetime);
 
   if (time_ptr != NULL) 
     {
@@ -1926,6 +1960,23 @@ int sh_unix_set_io_limit (char * c)
   SL_RETURN( 0, _("sh_unix_set_io_limit"));
 }
 
+#include <setjmp.h>
+
+static jmp_buf proc_wait_timeout;
+
+static void sh_unix_proc_alarmhandle (int mysignal)
+{
+  struct  sigaction  def_act;
+  def_act.sa_handler = SIG_IGN;
+
+  if (mysignal == SIGALRM)   /* use 'signal' to avoid compiler warning */
+    {  
+      alarm(0);
+      sigaction (SIGALRM, &def_act, NULL);
+    }
+  longjmp(proc_wait_timeout, 1);
+}
+
 
 /* obtain file info
  */
@@ -1943,18 +1994,33 @@ int sh_unix_getinfo (int level,
   char        * name;
   char        * tmp;
   char        * tmp2;
+  char        * p;
   char        * linknamebuf;
   int           linksize;
 
   int get_the_fd (SL_TICKET ticket);
 
   SL_TICKET     rval_open;
+  SL_TICKET     rval_open_2;
   int           fd;
   int           fstat_return;
 
   time_t        tend;
   time_t        tstart;
 
+  struct  sigaction  def_act;
+  struct  sigaction  new_act;
+  sigset_t           unblock;
+
+  def_act.sa_handler = SIG_IGN;
+  new_act.sa_handler = sh_unix_proc_alarmhandle;
+
+  sigemptyset( &new_act.sa_mask );       /* set an empty mask       */
+  new_act.sa_flags = 0;                  /* init sa_flags           */
+
+  sigemptyset(&unblock);
+  sigaddset  (&unblock, SIGALRM);
+
   SL_ENTER(_("sh_unix_getinfo"));
 
   /* --- Stat the file, and get checksum. ---
@@ -1970,6 +2036,8 @@ int sh_unix_getinfo (int level,
   else
     rval_open = -1;
 
+  rval_open_2 = rval_open;
+
   if (SL_ISERROR(rval_open))
     fd = -1;
   else
@@ -2039,9 +2107,30 @@ int sh_unix_getinfo (int level,
       if (fileHash != NULL)
 	{
 	  tiger_fd = rval_open;
+	  p = theFile->fullpath;
+
+	  if (setjmp(proc_wait_timeout) != 0) {
+	    TPT((0, FIL__, __LINE__, _("msg=<proc_wait: timeout>\n"))); 
+	    sl_strlcpy(fileHash, 
+		       _("000000000000000000000000000000000000000000000000"),
+		       KEY_LEN + 1);
+	    alarm(0);
+	    sigaction (SIGALRM, &def_act, NULL);
+	    sigprocmask(SIG_UNBLOCK, &unblock, NULL);
+	    goto time_out_door;
+	  }
+	  alarm(0);
+	  sigaction (SIGALRM, &new_act, &def_act);
+	  alarm(120);
+
 	  sl_strlcpy(fileHash, 
 		     sh_tiger_generic_hash (theFile->fullpath, TIGER_FD, 0), 
 		     KEY_LEN + 1);
+	  p = theFile->fullpath;
+
+	  alarm(0);
+	  sigaction (SIGALRM, &def_act, NULL);
+
 	}
     }
 
@@ -2049,14 +2138,34 @@ int sh_unix_getinfo (int level,
    */
   else if (fstat_return == 0 && S_ISREG(fbuf.st_mode))
     {
+      memcpy (&buf, &fbuf, sizeof( struct stat ));
+      p = theFile->fullpath;
       if (fileHash != NULL)
 	{
 	  tiger_fd = rval_open;
+
+	  if (setjmp(proc_wait_timeout) != 0) {
+	    TPT((0, FIL__, __LINE__, _("msg=<proc_wait: timeout>\n"))); 
+	    sl_strlcpy(fileHash, 
+		       _("000000000000000000000000000000000000000000000000"),
+		       KEY_LEN + 1);
+	    alarm(0);
+	    sigaction (SIGALRM, &def_act, NULL);
+	    goto time_out_door;
+	  }
+	  alarm(0);
+	  sigaction (SIGALRM, &new_act, &def_act);
+	  alarm(120);
+	  
 	  sl_strlcpy(fileHash, 
 		     sh_tiger_generic_hash (theFile->fullpath, TIGER_FD, 0), 
 		     KEY_LEN + 1);
+	  p = theFile->fullpath;
+
+	  alarm(0);
+	  sigaction (SIGALRM, &def_act, NULL);
+	  
 	}
-      memcpy (&buf, &fbuf, sizeof( struct stat ));
     }
 
   /* ---  case 4: a regular file, fstat failed --- 
@@ -2075,7 +2184,10 @@ int sh_unix_getinfo (int level,
 		       tmp2);
       SH_FREE(tmp2);
     }	  
-      
+
+ time_out_door:
+  rval_open = rval_open_2;
+
   if (!SL_ISERROR(rval_open))
     sl_close(rval_open);
 
@@ -2108,7 +2220,8 @@ int sh_unix_getinfo (int level,
   for (i = 0; i < 12; ++i) theFile->c_attributes[i] = '-';
   theFile->c_attributes[12] = '\0';
   theFile->attributes      =    0;
-  if (theFile->c_mode[0] != 'c' && theFile->c_mode[0] != 'b')
+  if (theFile->c_mode[0] != 'c' && theFile->c_mode[0] != 'b' &&
+      theFile->c_mode[0] != 'l' )
     sh_unix_getinfo_attr(theFile->fullpath, 
 			 &theFile->attributes, theFile->c_attributes);
 #endif
@@ -2384,7 +2497,13 @@ int sh_unix_testlock (char * filename)
 		}
 	    }
 	  else
-	    sh.flag.islocked = BAD;
+	    {
+	      sh_error_handle ((-1), FIL__, __LINE__, status,
+			       MSG_E_SUBGEN, 
+			       _("Cannot remove stale PID file, PID may be a running process"),
+			       _("sh_unix_testlock"));
+	      sh.flag.islocked = BAD;
+	    }
 	}
     }
   SL_RETURN((-1),_("sh_unix_testlock"));
@@ -2775,6 +2894,7 @@ int sh_derr ()
 
   p1 = getppid();
 
+
 #if defined(PTRACE_ATTACH) && defined(PTRACE_DETACH)
 
   if (0 == (p2 = fork()))
@@ -2795,9 +2915,15 @@ int sh_derr ()
 #endif
 
       ptrace(PTRACE_DETACH, p2, 0, 0);
+
+      /* need to send SIGCONT to make sure the parent process gets
+       * out of stopped state (one more ugly kludge)
+       */
+      kill(p2, SIGCONT);
       _exit(EXIT_SUCCESS);
     }
-  waitpid (p2, &p1, 0);
+  if (p2 != (-1))
+    waitpid (p2, &p1, 0);
 
 #elif defined(PT_ATTACH) && defined(PT_DETACH)
 
@@ -2832,12 +2958,18 @@ int sh_derr ()
 #ifdef __HP_UX
       ptrace(PT_DETACH, p2, 1, 0, 0);
 #else 
-      ptrace(PT_DETACH, p2, (char *) 1, 0);
+      ptrace(PT_DETACH,   p2, (char *) 1, 0);
 #endif
 
+      /* need to send SIGCONT to make sure the parent process gets
+       * out of stopped state (one more ugly kludge)
+       */
+      kill(p2, SIGCONT);
       _exit(EXIT_SUCCESS);
     }
-  waitpid (p2, &p1, 0);
+
+  if (p2 != (-1))
+    waitpid (p2, &p1, 0);
 
   /* defined(PT_ATTACH) && defined(PT_DETACH) */
 #endif
