@@ -104,13 +104,27 @@
 #endif 
 
 #ifndef UT_LINESIZE
-#define UT_LINESIZE           12
+#ifndef __UT_LINESIZE
+#define UT_LINESIZE            12
+#else
+#define UT_LINESIZE __UT_LINESIZE
+#endif 
 #endif
+
 #ifndef UT_NAMESIZE
-#define UT_NAMESIZE            8
+#ifndef __UT_NAMESIZE
+#define UT_NAMESIZE             8
+#else
+#define UT_NAMESIZE __UT_NAMESIZE
 #endif
+#endif
+
 #ifndef UT_HOSTSIZE
-#define UT_HOSTSIZE           16
+#ifndef __UT_HOSTSIZE
+#define UT_HOSTSIZE            16
+#else
+#define UT_HOSTSIZE __UT_HOSTSIZE
+#endif
 #endif
 
 #ifdef HAVE_UTMPX_H
@@ -620,7 +634,6 @@ static void sh_utmp_addlogin (struct SH_UTMP_S * ut)
 #endif
       user->time = ut->ut_time;
 
-
       if (username == NULL                              /* not yet logged in */
 	  || 0 == sl_strncmp(ut->ut_line, _("ttyp"), 4) /* in virt. console  */
 	  || 0 == sl_strncmp(ut->ut_line, _("ttyq"), 4) /* in virt. console  */
